package jtriton;

import java.io.FileWriter;
import java.io.IOException;

public class KMLWriter extends FileWriter {
	
	int indent = 0;
	
	public KMLWriter(String fileName, String kmlName) throws IOException {
		super(fileName);
		super.write(
				"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + 
				"<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n" +
				"<Folder>\n" +
				"  <name>" + kmlName + "</name>\n" +
				"  <open>1</open>\n" +
				"  <description>\n" +
				"    None\n" +
				"  </description>\n"
		);
		indent = 1;
	}
	
	public void close() throws IOException {
		super.write("</Folder>\n</kml>\n");
		super.close();
	}
	
	public void writeln(String str) throws IOException {
		for (int i = 0; i < indent; i++)
			super.write("  ");
		super.write(str + "\n");
	}
	
	public void writeln() throws IOException {
		super.write("\n");
	}
	
	public void pathStart(String pathName) throws IOException {
		writeln("<Placemark>");
		indent++;
		writeln("<name>" + pathName + "</name>");
		writeln("<LineString>");
		indent++;
		writeln("<tessellate>1</tessellate>");
		writeln("<coordinates>");
		indent++;
	}
	
	public void pathEnd() throws IOException {
		indent--;
		writeln("</coordinates>");
		indent--;
		writeln("</LineString>");
		indent--;
		writeln("</Placemark>");
	}
	
	public void pathPoint(double latitude, double longitude, double altitude) throws IOException {
		String out = String.format("%.6f,%.6f,%.1f ", longitude, latitude, altitude);
		super.write(out);
	}
	
	/*
	public static void main(String[] args) {
		try {
			KMLWriter f = new KMLWriter("test", "ObjectName");
			f.writeln("This is a test line");
			f.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	*/

}
