package jtriton;

public class NMEAdata {

	int		HH, MM;
	double	SS;
	double	latitude;
	double	longitude;
	double	altitude;
	double	geoidHeight;

	double	knots;
	double	course;

	double	trackMadeGood;
	double	groundSpeedKnots;
	double	groundSpeedKmh;

	int		day;
	int		month;
	int		year;

	int		fixQuality;
	int		numberOfSatellites;
	int		mode;

	int		lineNumber = 0;

	double	HDOP, VDOP, PDOP;

	final String[] head = {
			"$GPGGA",
			"$GPGSA",
			"$GPGSV",
			"$GPRMC",
			"$GPVTG",
	};

	public void reset() {
		latitude = 0.0;
		longitude = 0.0;
		fixQuality = 0;
		numberOfSatellites = 0;
		HDOP = 0.0;
		VDOP = 0.0;
		PDOP = 0.0;
		altitude = 0.0;
		geoidHeight = 0.0;
		knots = 0.0;
		course = 0.0;
		trackMadeGood = 0.0;
		groundSpeedKnots = 0.0;
		groundSpeedKmh = 0.0;
		mode = 1;
	}

	public NMEAdata () {
		reset();
		HH = 0;
		MM = 0;
		SS = 0.0;
		day = 0;
		month = 0;
		year = 0;
	}

	public boolean parse (String str) {
		lineNumber++;
		String[] chunk0 = str.split("\\*");
		if (chunk0.length != 2) {	// incomplete string
			System.err.printf("Wrong format in line  %6d: %s\n", lineNumber, str);
			return false;
		}
		int chsum1 = 0;
		for (int i = 1; i < chunk0[0].length(); i++)
			chsum1 ^= (int)(chunk0[0].charAt(i));
		chsum1 = chsum1 & 0xff;
		
		try {
			int chsum2 = Integer.valueOf(chunk0[1], 16);
			if (chsum1 != chsum2) {
				System.err.printf("Bad checksum in line  %6d: %s\n", lineNumber, str);
				return false;
			}

			String[] c = chunk0[0].split(",");
			int select = 0;
			for (; select < head.length; select++) {
				if (c[0].equals(head[select]))
					break;
			}
			boolean parsed = false;
			double time = 3600 * HH + 60 * MM + SS;
			switch (select) {
			case 0:
				fixQuality = Integer.valueOf(c[6]);
				numberOfSatellites = Integer.valueOf(c[7]);
				if (fixQuality > 0) {
					latitude = Double.valueOf(c[2].substring(0,2)) 
					+ Double.valueOf(c[2].substring(2))/60.0;
					if (c[3].charAt(0) == 'S')
						latitude = -latitude;
					longitude = Double.valueOf(c[4].substring(0,3)) 
					+ Double.valueOf(c[4].substring(3))/60.0;
					if (c[5].charAt(0) == 'W')
						longitude = -longitude;
					HDOP = Double.valueOf(c[8]);
					altitude = Double.valueOf(c[9]);
					if (c[10].charAt(0) == 'F')
						altitude = altitude / 0.305; 
					geoidHeight = Double.valueOf(c[11]);
					if (c[12].charAt(0) == 'F')
						geoidHeight = geoidHeight / 0.305;
				} else
					reset();
				if (c[1].length() >= 6) {
					HH = Integer.valueOf(c[1].substring(0,2));
					MM = Integer.valueOf(c[1].substring(2,4));
					SS = Double.valueOf(c[1].substring(4));
				}
				parsed = true;
				break;
			case 1:
				mode = Integer.valueOf(c[2]);
				if (mode > 1) {
					PDOP = Double.valueOf(c[15]);
					HDOP = Double.valueOf(c[16]);
					VDOP = Double.valueOf(c[17]);
				} else
					reset();
				parsed = true;
				break;
			case 2:
				break;
			case 3:
				if (c[2].charAt(0) == 'A') {
					latitude = Double.valueOf(c[3].substring(0,2)) 
					+ Double.valueOf(c[3].substring(2))/60.0;
					if (c[4].charAt(0) == 'S')
						latitude = -latitude;
					longitude = Double.valueOf(c[5].substring(0,3)) 
					+ Double.valueOf(c[5].substring(3))/60.0;
					if (c[6].charAt(0) == 'W')
						longitude = -longitude;
					knots =  Double.valueOf(c[7]);
					course =  Double.valueOf(c[8]);
				} else
					reset();
				if (c[1].length() >= 6) {
					HH = Integer.valueOf(c[1].substring(0,2));
					MM = Integer.valueOf(c[1].substring(2,4));
					SS = Double.valueOf(c[1].substring(4));
				}
				if (c[9].length() == 6) {
					day = Integer.valueOf(c[9].substring(0,2));
					month = Integer.valueOf(c[9].substring(2,4));
					year = Integer.valueOf(c[9].substring(4));
					if (year < 90)
						year += 2000;
					else
						year += 1900;
				}
				parsed = true;
				break;
			case 4:
				if (c[1].length() >= 1)
					trackMadeGood = Double.valueOf(c[1]);
				if (c[5].length() >= 1 && c[6].charAt(0) == 'N')
					groundSpeedKnots = Double.valueOf(c[5]);
				if (c[7].length() >= 1 && c[8].charAt(0) == 'K')
					groundSpeedKmh = Double.valueOf(c[7]);
				parsed = true;
				break;
			default:
				System.err.println("Unrecognized header: " + c[0]);
				return false;
			}
			if (parsed) {
				double newTime = 3600 * HH + 60 * MM + SS;
				//System.out.println(str);
				if (newTime != time) {
					boolean result = (fixQuality > 0) && (mode > 1);
					/*
				if (result)
					System.out.printf("%04d.%02d.%02d %02d:%02d:%06.3f %11.6f %11.6f %d %2d (%d: %3.1f %3.1f %3.1f) %7.1f %5.1f %6.2f %6.2f %6.2f %6.2f %6.2f\n", 
						year, month, day, HH, MM, SS, latitude, longitude, fixQuality,
						numberOfSatellites, mode, HDOP, VDOP, PDOP, altitude, geoidHeight, 
						knots, groundSpeedKnots, groundSpeedKmh, course, trackMadeGood);
					 */
					return result;
				}
			}
		} catch (NumberFormatException e) {
			System.err.printf("Error in line         %6d: %s\n", lineNumber, str);
			//e.printStackTrace();
		}
		return false;
	}

}
